#!/bin/sh

# All Rights Reserved * Licensed Materials - Property of IBM
# 5724-I63, 5724-H88, 5655-N01, 5733-W60 (C) COPYRIGHT International Business Machines Corp., 1997,2011
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.

# WebSphere Virtual Enterprise scripts to link two cells together for OVERLAY communication 

binDir=`dirname $0`
. $binDir/setupCmdLine.sh

if [ $# -ne 2 ] ; then
    echo "Usage: unlinkCells.sh <cell1Info> <cell2Info>"
    echo "   where both <cell1Info> and <cell2Info> are of one of the following two forms:"
    echo "     <dmgrHost>:<dmgrSoapPort>"
    echo "        if WebSphere security is NOT enabled in the cell"
    echo "     <dmgrHost>:<dmgrSoapPort>:<userName>:<password>"
    echo "        if WebSphere security is enabled in the cell"
    echo "This script unlinks two cells previously linked together."
    exit 1
fi

HOST1=`echo $1 | cut -d: -f1`
PORT1=`echo $1 | cut -d: -f2`
UID1=`echo $1 | cut -d: -f3`
PWD1=`echo $1 | cut -d: -f4`

HOST2=`echo $2 | cut -d: -f1`
PORT2=`echo $2 | cut -d: -f2`
UID2=`echo $2 | cut -d: -f3`
PWD2=`echo $2 | cut -d: -f4`

PY="-f ${WAS_HOME}/bin/linkCells.py"
UNLINKPY="-f ${WAS_HOME}/bin/importOverlayConfig.py unlink"

INVOKE1="${WAS_HOME}/bin/wsadmin.sh -conntype SOAP -host $HOST1 -port $PORT1 -user $UID1 -password $PWD1 ${PY}"
INVOKE2="${WAS_HOME}/bin/wsadmin.sh -conntype SOAP -host $HOST2 -port $PORT2 -user $UID2 -password $PWD2 ${PY}"
INVOKE1UNLINK="${WAS_HOME}/bin/wsadmin.sh -conntype SOAP -host $HOST1 -port $PORT1 -user $UID1 -password $PWD1 ${UNLINKPY}"
INVOKE2UNLINK="${WAS_HOME}/bin/wsadmin.sh -conntype SOAP -host $HOST2 -port $PORT2 -user $UID2 -password $PWD2 ${UNLINKPY}"

OVERLAY_FILE1=/tmp/_overlay1_$$
OVERLAY_FILE2=/tmp/_overlay2_$$

echo "Begin unlinking cells ..."

#Copy the overlay config from cell2 to cell1 as OVERLAY_FILE2
#Execute command to remove cell2 nodes

$INVOKE2 -getOverlay::${OVERLAY_FILE2} 
$INVOKE1UNLINK ${OVERLAY_FILE2}
if [ "$PWD1" != "" ]; then
    $INVOKE2 -removeSigner:${HOST1}:${PORT1}
fi
# Copy the overlay config from cell1 to cell2 as OVERLAY_FILE1
#Execuate command to remove cell1 nodes
$INVOKE1 -getOverlay::${OVERLAY_FILE1} 
$INVOKE1UNLINK ${OVERLAY_FILE2}
# If a password was provided for cell2, import cell2's signer certificate into cell1
if [ "$PWD2" != "" ]; then
    $INVOKE1 -removeSigner:${HOST2}:${PORT2}
fi

# Clean up temporary files
rm -f $OVERLAY_FILE1 $OVERLAY_FILE2

echo "Done unlinking cells."
